@@ -1,15 +1,7 @@
+from punpy.digital_effects_table.measurement_function import MeasurementFunction
 from punpy.lpu.lpu_propagation import LPUPropagation
 from punpy.mc.mc_propagation import MCPropagation
-from punpy.utilities.utilities import (
-    calculate_Jacobian,
-    calculate_flattened_corr,
-    separate_flattened_corr,
-    convert_corr_to_cov,
-    convert_cov_to_corr,
-    correlation_from_covariance,
-    uncertainty_from_covariance,
-    nearestPD_cholesky,
-)
+
 from ._version import get_versions
 
 """___Authorship___"""
