@@ -1,10 +1,11 @@
 """Use Monte Carlo to propagate uncertainties"""
 
+import time
 import warnings
 from multiprocessing import Pool
 
+import comet_maths as cm
 import numpy as np
-
 import punpy.utilities.utilities as util
 
 """___Authorship___"""
@@ -16,7 +17,7 @@ __status__ = "Development"
 
 
 class MCPropagation:
-    def __init__(self, steps, parallel_cores=0, dtype=None):
+    def __init__(self, steps, parallel_cores=0, dtype=None, verbose=False):
         """
         Initialise MC Propagator
 
@@ -26,6 +27,8 @@ class MCPropagation:
         :type parallel_cores: int
         :param dtype: numpy dtype for output variables
         :type dtype: numpy dtype
+        :param verbose: bool to set if logging info should be printed
+        :type verbose: bool
         """
 
         self.MCsteps = steps
@@ -33,6 +36,8 @@ class MCPropagation:
         self.dtype = dtype
         if parallel_cores > 1:
             self.pool = Pool(self.parallel_cores)
+        self.verbose = verbose
+        self.starttime = time.time()
 
     def propagate_random(
         self,
@@ -42,6 +47,7 @@ class MCPropagation:
         corr_x=None,
         param_fixed=None,
         corr_between=None,
+        samples=None,
         return_corr=False,
         return_samples=False,
         repeat_dims=-99,
@@ -101,6 +107,7 @@ class MCPropagation:
             corr_x,
             param_fixed=param_fixed,
             corr_between=corr_between,
+            samples=samples,
             return_corr=return_corr,
             return_samples=return_samples,
             repeat_dims=repeat_dims,
@@ -119,6 +126,7 @@ class MCPropagation:
         corr_x=None,
         param_fixed=None,
         corr_between=None,
+        samples=None,
         return_corr=False,
         return_samples=False,
         repeat_dims=-99,
@@ -178,6 +186,7 @@ class MCPropagation:
             corr_x,
             param_fixed=param_fixed,
             corr_between=corr_between,
+            samples=samples,
             return_corr=return_corr,
             return_samples=return_samples,
             repeat_dims=repeat_dims,
@@ -196,6 +205,7 @@ class MCPropagation:
         corr_x,
         param_fixed=None,
         corr_between=None,
+        samples=None,
         return_corr=False,
         return_samples=False,
         repeat_dims=-99,
@@ -231,7 +241,7 @@ class MCPropagation:
         :type repeat_dims: integer or list of 2 integers, optional
         :param corr_axis: set to positive integer to select the axis used in the correlation matrix. The correlation matrix will then be averaged over other dimensions. Defaults to -99, for which the input array will be flattened and the full correlation matrix calculated.
         :type corr_axis: integer, optional
-        :param fixed_corr_var: set to integer to copy the correlation matrix of the dimiension the integer refers to. Set to True to automatically detect if only one uncertainty is present and the correlation matrix of that dimension should be copied. Defaults to False.
+        :param fixed_corr_var: set to integer to copy the correlation matrix of the dimension the integer refers to. Set to True to automatically detect if only one uncertainty is present and the correlation matrix of that dimension should be copied. Defaults to False.
         :type fixed_corr_var: bool or integer, optional
         :param output_vars: number of output parameters in the measurement function. Defaults to 1.
         :type output_vars: integer, optional
@@ -242,6 +252,11 @@ class MCPropagation:
         :return: uncertainties on measurand
         :rtype: array
         """
+        if self.verbose:
+            print(
+                "starting propagation (%s s since creation of prop object)"
+                % (time.time() - self.starttime)
+            )
         (
             yshapes,
             x,
@@ -276,6 +291,7 @@ class MCPropagation:
                 corr_x,
                 param_fixed,
                 corr_between,
+                samples,
                 return_corr,
                 return_samples,
                 -99,
@@ -325,6 +341,7 @@ class MCPropagation:
                     corr_x,
                     param_fixed,
                     corr_between,
+                    samples,
                     return_corr,
                     return_samples,
                     -99,
@@ -348,6 +365,11 @@ class MCPropagation:
                     output_vars,
                     refyvar,
                 )
+                if self.verbose:
+                    print(
+                        "repeated measurement %s out of %s processed (%s s since creation of prop object)"
+                        % (i, n_repeats, time.time() - self.starttime)
+                    )
 
             outs = self.finish_repeated_outs(
                 outs,
@@ -365,19 +387,20 @@ class MCPropagation:
             return outs
 
         else:
-            MC_data = np.empty(len(x), dtype=np.ndarray)
-            for i in range(len(x)):
-                if not hasattr(x[i], "__len__"):
-                    MC_data[i] = self.generate_samples_systematic(x[i], u_x[i])
-                elif corr_x[i] == "rand":
-                    MC_data[i] = self.generate_samples_random(x[i], u_x[i])
-                elif corr_x[i] == "syst":
-                    MC_data[i] = self.generate_samples_systematic(x[i], u_x[i])
-                else:
-                    MC_data[i] = self.generate_samples_correlated(x, u_x, corr_x, i)
+            if samples is not None:
+                MC_data = samples
+            else:
+                MC_data = np.empty(len(x), dtype=np.ndarray)
 
-            if corr_between is not None:
-                MC_data = self.correlate_samples_corr(MC_data, corr_between)
+                for i in range(len(x)):
+                    MC_data[i] = cm.generate_sample(self.MCsteps, x, u_x, corr_x, i)
+                if corr_between is not None:
+                    MC_data = cm.correlate_sample_corr(MC_data, corr_between)
+                if self.verbose:
+                    print(
+                        "samples generated (%s s since creation of prop object)"
+                        % (time.time() - self.starttime)
+                    )
 
             return self.process_samples(
                 func,
@@ -578,29 +601,31 @@ class MCPropagation:
             MC_data = np.empty(len(x), dtype=np.ndarray)
             for i in range(len(x)):
                 if not hasattr(x[i], "__len__"):
-                    MC_data[i] = self.generate_samples_systematic(x[i], cov_x[i])
+                    MC_data[i] = cm.generate_sample_systematic(
+                        self.MCsteps, x[i], cov_x[i]
+                    )
                 elif param_fixed is not None:
                     if param_fixed[i] and (len(x[i].shape) == 2):
                         MC_data[i] = np.array(
                             [
-                                self.generate_samples_cov(
-                                    x[i][:, j].flatten(), cov_x[i]
+                                cm.generate_sample_cov(
+                                    self.MCsteps, x[i][:, j].ravel(), cov_x[i]
                                 ).reshape(x[i][:, j].shape + (self.MCsteps,))
                                 for j in range(x[i].shape[1])
                             ]
                         ).T
                         MC_data[i] = np.moveaxis(MC_data[i], 0, 1)
                     else:
-                        MC_data[i] = self.generate_samples_cov(
-                            x[i].flatten(), cov_x[i]
+                        MC_data[i] = cm.generate_sample_cov(
+                            self.MCsteps, x[i].ravel(), cov_x[i]
                         ).reshape(x[i].shape + (self.MCsteps,))
                 else:
-                    MC_data[i] = self.generate_samples_cov(
-                        x[i].flatten(), cov_x[i]
+                    MC_data[i] = cm.generate_sample_cov(
+                        self.MCsteps, x[i].ravel(), cov_x[i]
                     ).reshape(x[i].shape + (self.MCsteps,))
 
             if corr_between is not None:
-                MC_data = self.correlate_samples_corr(MC_data, corr_between)
+                MC_data = cm.correlate_sample_corr(MC_data, corr_between)
 
         return self.process_samples(
             func,
@@ -725,7 +750,7 @@ class MCPropagation:
                     if not isinstance(corr_x[i], str):
                         if np.any(corr_x[i] > 1.000001):
                             raise ValueError(
-                                "One of the provided correlation matrices "
+                                "punpy.mc_propagation: One of the provided correlation matrices "
                                 "has elements >1."
                             )
 
@@ -753,8 +778,9 @@ class MCPropagation:
                 if corr_axis > repeat_dims[0]:
                     corr_axis -= 1
                 elif corr_axis == repeat_dims[0]:
-                    print("corr_axis and repeat_axis keywords should not be the same.")
-                    exit()
+                    raise ValueError(
+                        "punpy.mc_propagation: corr_axis and repeat_axis keywords should not be the same."
+                    )
             else:
                 n_repeats = 0
             repeat_shape = (n_repeats,)  # repeat_dims = -99
@@ -765,14 +791,15 @@ class MCPropagation:
             if corr_axis > repeat_dims[0]:
                 corr_axis -= 1
             elif corr_axis == repeat_dims[0]:
-                print("corr_axis and repeat_axis keywords should not be the same.")
-                exit()
+                raise ValueError(
+                    "punpy.mc_propagation: corr_axis and repeat_axis keywords should not be the same."
+                )
             if corr_axis > repeat_dims[1]:
                 corr_axis -= 1
             elif corr_axis == repeat_dims[1]:
-                print("corr_axis and repeat_axis keywords should not be the same.")
-                exit()
-
+                raise ValueError(
+                    "punpy.mc_propagation: corr_axis and repeat_axis keywords should not be the same."
+                )
         return (
             yshapes,
             x,
@@ -1047,8 +1074,16 @@ class MCPropagation:
                             for ii in range(yshapes[i][0]):
                                 for iii in range(yshapes[i][1]):
                                     u_func[i][ii, iii] = u_funcb[i][iii][ii]
+                        elif len(yshapes[i]) == 2 and len(u_funcb[i].shape) == 3:
+                            if yshapes[i][0] == u_funcb[i].shape[0]:
+                                for ii in range(yshapes[i][0]):
+                                    for iii in range(yshapes[i][1]):
+                                        u_func[i][ii, iii] = u_funcb[i][ii][0][iii]
                         else:
-                            print("this shape is not supported")
+                            print(yshapes, i, u_funcb[i].shape)
+                            raise ValueError(
+                                "punpy.mc_propagation: this shape is not supported"
+                            )
 
         else:
             if output_vars == 1:
@@ -1069,7 +1104,7 @@ class MCPropagation:
         ):
             print(u_func.shape, u_func[0].shape, yshapes)
             raise ValueError(
-                "The shape of the uncertainties does not match the shape"
+                "punpy.mc_propagation: The shape of the uncertainties does not match the shape"
                 "of the measurand. This is likely a problem with combining"
                 "repeated measurements (repeat_dims keyword)."
             )
@@ -1082,15 +1117,15 @@ class MCPropagation:
             if return_corr:
                 outs[1] = outs[1] / n_repeats
                 if output_vars == 1:
-                    if not util.isPD(outs[1]):
-                        outs[1] = util.nearestPD_cholesky(
+                    if not cm.isPD(outs[1]):
+                        outs[1] = cm.nearestPD_cholesky(
                             outs[1], corr=True, return_cholesky=False
                         )
 
                 else:
                     for j in range(output_vars):
-                        if not util.isPD(outs[1][j]):
-                            outs[1][j] = util.nearestPD_cholesky(
+                        if not cm.isPD(outs[1][j]):
+                            outs[1][j] = cm.nearestPD_cholesky(
                                 outs[1][j], corr=True, return_cholesky=False
                             )
 
@@ -1098,8 +1133,8 @@ class MCPropagation:
                         [yshapes[i] == yshapes[refyvar] for i in range(len(yshapes))]
                     ):
                         outs[2] = outs[2] / n_repeats
-                        if not util.isPD(outs[2]):
-                            outs[2] = util.nearestPD_cholesky(
+                        if not cm.isPD(outs[2]):
+                            outs[2] = cm.nearestPD_cholesky(
                                 outs[2], corr=True, return_cholesky=False
                             )
         return outs
@@ -1188,7 +1223,9 @@ class MCPropagation:
                                             ii, iii, iiii
                                         ]
                     else:
-                        print("this shape is not supported")
+                        raise ValueError(
+                            "punpy.mc_propagation: this shape is not supported"
+                        )
 
         else:
             # We again need to reorder the input quantities samples in order to be able to pass them to p.starmap
@@ -1204,6 +1241,12 @@ class MCPropagation:
                 for i in range(output_vars):
                     MC_y[i] = np.moveaxis(MC_y2[:, i], 0, -1)
 
+        if self.verbose:
+            print(
+                "samples propagated (%s s since creation of prop object)"
+                % (time.time() - self.starttime)
+            )
+
         # if hasattr(MC_y[0,0], '__len__'):
         #     print(yshape,np.array(MC_y[0,0]).shape,np.array(MC_y[1,0]).shape,np.array(MC_y[2,0]).shape,np.array(MC_y[3,0]).shape)
         if output_vars == 1:
@@ -1211,8 +1254,15 @@ class MCPropagation:
         else:
             u_func = np.empty(output_vars, dtype=object)
             for i in range(output_vars):
+                # print(MC_y[i].shape,MC_y[i])
                 u_func[i] = np.std(np.array(MC_y[i]), axis=-1, dtype=self.dtype)
 
+        if self.verbose:
+            print(
+                "std calculated (%s s since creation of prop object)"
+                % (time.time() - self.starttime)
+            )
+
         if not return_corr:
             if return_samples:
                 return u_func, MC_y, data
@@ -1221,14 +1271,21 @@ class MCPropagation:
         else:
             if output_vars == 1:
                 if fixed_corr is None:
-                    corr_y = self.calculate_corr(MC_y, corr_axis).astype(self.dtype)
+                    corr_y = cm.calculate_corr(MC_y, corr_axis).astype(self.dtype)
                     if PD_corr:
-                        if not util.isPD(corr_y):
-                            corr_y = util.nearestPD_cholesky(
+                        if not cm.isPD(corr_y):
+                            corr_y = cm.nearestPD_cholesky(
                                 corr_y, corr=True, return_cholesky=False
                             )
                 else:
                     corr_y = fixed_corr
+
+                if self.verbose:
+                    print(
+                        "corr calculated (%s s since creation of prop object)"
+                        % (time.time() - self.starttime)
+                    )
+
                 if return_samples:
                     return u_func, corr_y, MC_y, data
                 else:
@@ -1239,12 +1296,12 @@ class MCPropagation:
                 corr_ys = np.empty(output_vars, dtype=object)
                 for i in range(output_vars):
                     if fixed_corr is None:
-                        corr_ys[i] = self.calculate_corr(MC_y[i], corr_axis).astype(
+                        corr_ys[i] = cm.calculate_corr(MC_y[i], corr_axis).astype(
                             self.dtype
                         )
                         if PD_corr:
-                            if not util.isPD(corr_ys[i]):
-                                corr_ys[i] = util.nearestPD_cholesky(
+                            if not cm.isPD(corr_ys[i]):
+                                corr_ys[i] = cm.nearestPD_cholesky(
                                     corr_ys[i], corr=True, return_cholesky=False
                                 )
                     else:
@@ -1258,290 +1315,18 @@ class MCPropagation:
                     corr_out = None
 
                 if PD_corr and corr_out is not None:
-                    if not util.isPD(corr_out):
-                        corr_out = util.nearestPD_cholesky(
+                    if not cm.isPD(corr_out):
+                        corr_out = cm.nearestPD_cholesky(
                             corr_out, corr=True, return_cholesky=False
                         )
+
+                if self.verbose:
+                    print(
+                        "corr calculated for output_var>1 (%s s since creation of prop object)"
+                        % (time.time() - self.starttime)
+                    )
+
                 if return_samples:
                     return u_func, corr_ys, corr_out, MC_y, data
                 else:
                     return u_func, corr_ys, corr_out
-
-    def calculate_corr(self, MC_y, corr_axis=-99):
-        """
-        Calculate the correlation matrix between the MC-generated samples of output quantities.
-        If corr_axis is specified, this axis will be the one used to calculate the correlation matrix (e.g. if corr_axis=0 and x.shape[0]=n, the correlation matrix will have shape (n,n)).
-        This will be done for each combination of parameters in the other dimensions and the resulting correlation matrices are averaged.
-
-        :param MC_y: MC-generated samples of the output quantities (measurands)
-        :type MC_y: array
-        :param corr_axis: set to positive integer to select the axis used in the correlation matrix. The correlation matrix will then be averaged over other dimensions. Defaults to -99, for which the input array will be flattened and the full correlation matrix calculated.
-        :type corr_axis: integer, optional
-        :return: correlation matrix
-        :rtype: array
-        """
-        # print("the shape is:",MC_y.shape)
-
-        if len(MC_y.shape) < 3:
-            corr_y = np.corrcoef(MC_y)
-
-        elif len(MC_y.shape) == 3:
-            if corr_axis == 0:
-                corr_ys = np.zeros(
-                    (len(MC_y[:, 0, 0]), len(MC_y[:, 0, 0])), dtype=self.dtype
-                )
-                for i in range(len(MC_y[0])):
-                    corr_ys += np.corrcoef(MC_y[:, i])
-                corr_y = corr_ys / len(MC_y[0])
-
-            elif corr_axis == 1:
-                corr_ys = np.zeros(
-                    (len(MC_y[0, :, 0]), len(MC_y[0, :, 0])), dtype=self.dtype
-                )
-                # corr_ys = np.zeros(MC_y[0].shape)
-                for i in range(len(MC_y)):
-                    corr_ys += np.corrcoef(MC_y[i])
-                corr_y = corr_ys / len(MC_y)
-
-            else:
-                MC_y = MC_y.reshape((MC_y.shape[0] * MC_y.shape[1], self.MCsteps))
-                corr_y = np.corrcoef(MC_y)
-
-        elif len(MC_y.shape) == 4:
-            if corr_axis == 0:
-                corr_ys = np.zeros(
-                    (len(MC_y[:, 0, 0, 0]), len(MC_y[:, 0, 0, 0])), dtype=self.dtype
-                )
-                # corr_ys = np.zeros(MC_y[:, 0, 0].shape)
-                for i in range(len(MC_y[0])):
-                    for j in range(len(MC_y[0, 0])):
-                        corr_ys += np.corrcoef(MC_y[:, i, j])
-                corr_y = corr_ys / (len(MC_y[0]) * len(MC_y[0, 0]))
-
-            elif corr_axis == 1:
-                corr_ys = np.zeros(
-                    (len(MC_y[0, :, 0, 0]), len(MC_y[0, :, 0, 0])), dtype=self.dtype
-                )
-                # corr_ys = np.zeros(MC_y[0, :, 0].shape)
-                for i in range(len(MC_y)):
-                    for j in range(len(MC_y[0, 0])):
-                        corr_ys += np.corrcoef(MC_y[i, :, j])
-                corr_y = corr_ys / (len(MC_y) * len(MC_y[0, 0]))
-
-            elif corr_axis == 2:
-                corr_ys = np.zeros(
-                    (len(MC_y[0, 0, :, 0]), len(MC_y[0, 0, :, 0])), dtype=self.dtype
-                )
-                # corr_ys = np.zeros(MC_y[0, 0].shape)
-                for i in range(len(MC_y)):
-                    for j in range(len(MC_y[0])):
-                        corr_ys += np.corrcoef(MC_y[i, j])
-                corr_y = corr_ys / (len(MC_y) * len(MC_y[0]))
-            else:
-                MC_y = MC_y.reshape(
-                    (MC_y.shape[0] * MC_y.shape[1] * MC_y.shape[2], self.MCsteps)
-                )
-                corr_y = np.corrcoef(MC_y)
-        else:
-            print(
-                "MC_y has too high dimensions. Reduce the dimensionality of the input data"
-            )
-            exit()
-
-        return corr_y
-
-    def generate_samples_correlated(self, x, u_x, corr_x, i):
-        """
-        Generate correlated MC samples of input quantity with given uncertainties and correlation matrix.
-        Samples are generated using generate_samples_cov() after matching up the uncertainties to the right correlation matrix.
-        It is possible to provide one correlation matrix to be used for each measurement (which each have an uncertainty) or a correlation matrix per measurement.
-
-        :param x: list of input quantities (usually numpy arrays)
-        :type x: list[array]
-        :param u_x: list of uncertainties/covariances on input quantities (usually numpy arrays)
-        :type u_x: list[array]
-        :param corr_x: list of correlation matrices (n,n) along non-repeating axis, or list of correlation matrices for each repeated measurement.
-        :type corr_x: list[array], optional
-        :param i: index of the input quantity (in x)
-        :type i: int
-        :return: generated samples
-        :rtype: array
-        """
-        if len(x[i].shape) == 2:
-            if len(corr_x[i]) == len(u_x[i]):
-                MC_data = np.zeros((u_x[i].shape) + (self.MCsteps,))
-                for j in range(len(u_x[i][0])):
-                    cov_x = util.convert_corr_to_cov(corr_x[i], u_x[i][:, j])
-                    MC_data[:, j, :] = self.generate_samples_cov(
-                        x[i][:, j].flatten(), cov_x
-                    ).reshape(x[i][:, j].shape + (self.MCsteps,))
-            else:
-                MC_data = np.zeros((u_x[i].shape) + (self.MCsteps,))
-                for j in range(len(u_x[i][:, 0])):
-                    cov_x = util.convert_corr_to_cov(corr_x[i], u_x[i][j])
-                    MC_data[j, :, :] = self.generate_samples_cov(
-                        x[i][j].flatten(), cov_x
-                    ).reshape(x[i][j].shape + (self.MCsteps,))
-        else:
-            cov_x = util.convert_corr_to_cov(corr_x[i], u_x[i])
-            MC_data = self.generate_samples_cov(x[i].flatten(), cov_x).reshape(
-                x[i].shape + (self.MCsteps,)
-            )
-
-        return MC_data
-
-    def generate_samples_random(self, param, u_param):
-        """
-        Generate MC samples of input quantity with random (Gaussian) uncertainties.
-
-        :param param: values of input quantity (mean of distribution)
-        :type param: float or array
-        :param u_param: uncertainties on input quantity (std of distribution)
-        :type u_param: float or array
-        :return: generated samples
-        :rtype: array
-        """
-        if not hasattr(param, "__len__"):
-            return (
-                np.random.normal(size=self.MCsteps).astype(self.dtype) * u_param + param
-            )
-        elif len(param.shape) == 1:
-            return (
-                np.random.normal(size=(len(param), self.MCsteps)).astype(self.dtype)
-                * u_param[:, None]
-                + param[:, None]
-            )
-        elif len(param.shape) == 2:
-            return (
-                np.random.normal(size=param.shape + (self.MCsteps,))
-                * u_param[:, :, None]
-                + param[:, :, None]
-            )
-        elif len(param.shape) == 3:
-            return (
-                np.random.normal(size=param.shape + (self.MCsteps,)).astype(self.dtype)
-                * u_param[:, :, :, None]
-                + param[:, :, :, None]
-            )
-        else:
-            print("parameter shape not supported: ", param.shape, param)
-            exit()
-
-    def generate_samples_systematic(self, param, u_param):
-        """
-        Generate correlated MC samples of input quantity with systematic (Gaussian) uncertainties.
-
-        :param param: values of input quantity (mean of distribution)
-        :type param: float or array
-        :param u_param: uncertainties on input quantity (std of distribution)
-        :type u_param: float or array
-        :return: generated samples
-        :rtype: array
-        """
-        if not hasattr(param, "__len__"):
-            return (
-                np.random.normal(size=self.MCsteps).astype(self.dtype) * u_param + param
-            )
-        elif len(param.shape) == 1:
-            return (
-                np.dot(
-                    u_param[:, None],
-                    np.random.normal(size=self.MCsteps).astype(self.dtype)[None, :],
-                )
-                + param[:, None]
-            )
-        elif len(param.shape) == 2:
-            return (
-                np.dot(
-                    u_param[:, :, None],
-                    np.random.normal(size=self.MCsteps).astype(self.dtype)[
-                        :, None, None
-                    ],
-                )[:, :, :, 0]
-                + param[:, :, None]
-            )
-        elif len(param.shape) == 3:
-            return (
-                np.dot(
-                    u_param[:, :, :, None],
-                    np.random.normal(size=self.MCsteps).astype(self.dtype)[
-                        :, None, None, None
-                    ],
-                )[:, :, :, :, 0, 0]
-                + param[:, :, :, None]
-            )
-        else:
-            print("parameter shape not supported")
-            exit()
-
-    def generate_samples_cov(self, param, cov_param):
-        """
-        Generate correlated MC samples of input quantity with a given covariance matrix.
-        Samples are generated independent and then correlated using Cholesky decomposition.
-
-        :param param: values of input quantity (mean of distribution)
-        :type param: array
-        :param cov_param: covariance matrix for input quantity
-        :type cov_param: array
-        :return: generated samples
-        :rtype: array
-        """
-        try:
-            L = np.linalg.cholesky(cov_param)
-        except:
-            L = util.nearestPD_cholesky(cov_param)
-
-        return (
-            np.dot(
-                L, np.random.normal(size=(len(param), self.MCsteps)).astype(self.dtype)
-            )
-            + param[:, None]
-        )
-
-    def correlate_samples_corr(self, samples, corr):
-        """
-        Method to correlate independent samples of input quantities using correlation matrix and Cholesky decomposition.
-
-        :param samples: independent samples of input quantities
-        :type samples: array[array]
-        :param corr: correlation matrix between input quantities
-        :type corr: array
-        :return: correlated samples of input quantities
-        :rtype: array[array]
-        """
-        if np.max(corr) > 1.000001 or len(corr) != len(samples):
-            raise ValueError(
-                "The correlation matrix between variables is not the right shape or has elements >1."
-            )
-        else:
-            try:
-                L = np.array(np.linalg.cholesky(corr))
-            except:
-                L = util.nearestPD_cholesky(corr)
-
-            # Cholesky needs to be applied to Gaussian distributions with mean=0 and std=1,
-            # We first calculate the mean and std for each input quantity
-            means = np.array(
-                [np.mean(samples[i]) for i in range(len(samples))], dtype=self.dtype
-            )
-            stds = np.array(
-                [np.std(samples[i]) for i in range(len(samples))], dtype=self.dtype
-            )
-
-            # We normalise the samples with the mean and std, then apply Cholesky, and finally reapply the mean and std.
-            if all(stds != 0):
-                return np.dot(L, (samples - means) / stds) * stds + means
-
-            # If any of the variables has no uncertainty, the normalisation will fail. Instead we leave the parameters without uncertainty unchanged.
-            else:
-                samples_out = samples[:]
-                id_nonzero = np.where(stds != 0)
-                samples_out[id_nonzero] = (
-                    np.dot(
-                        L[id_nonzero][:, id_nonzero],
-                        (samples[id_nonzero] - means[id_nonzero]) / stds[id_nonzero],
-                    )[:, 0]
-                    * stds[id_nonzero]
-                    + means[id_nonzero]
-                )
-                return samples_out
