@@ -4,10 +4,9 @@ Tests for mc propagation class
 
 import unittest
 
+import comet_maths as cm
 import numpy as np
 import numpy.testing as npt
-
-import punpy.utilities.utilities as util
 from punpy.mc.mc_propagation import MCPropagation
 
 """___Authorship___"""
@@ -102,17 +101,17 @@ class TestMCPropagation(unittest.TestCase):
     """
 
     def test_propagate_random_1D(self):
-        prop = MCPropagation(20000, parallel_cores=0)
+        prop = MCPropagation(40000, parallel_cores=0)
 
         uf, ucorr = prop.propagate_random(function, xs, xerrs, return_corr=True)
         npt.assert_allclose(ucorr, np.eye(len(ucorr)), atol=0.06)
         npt.assert_allclose(uf, yerr_uncorr, rtol=0.06)
-        ucov = util.convert_corr_to_cov(ucorr, uf)
-        ucorr2 = util.convert_cov_to_corr(ucov, uf)
+        ucov = cm.convert_corr_to_cov(ucorr, uf)
+        ucorr2 = cm.convert_cov_to_corr(ucov, uf)
         npt.assert_allclose(ucorr, ucorr2, atol=0.01)
 
         uf = prop.propagate_random(function, xs, xerrs, corr_between=np.ones((2, 2)))
-        npt.assert_allclose(uf, yerr_corr, rtol=0.06)
+        npt.assert_allclose(uf, yerr_corr, atol=0.1)
 
         uf, ucorr, yvalues, xvalues = prop.propagate_random(
             function, xs, xerrs, return_corr=True, return_samples=True
@@ -123,7 +122,7 @@ class TestMCPropagation(unittest.TestCase):
         uf, yvalues, xvalues = prop.propagate_random(
             function, xs, xerrs, corr_between=np.ones((2, 2)), return_samples=True
         )
-        npt.assert_allclose(uf, yerr_corr, rtol=0.06)
+        npt.assert_allclose(uf, yerr_corr, atol=0.1)
 
         uf, yvalues, xvalues = prop.propagate_standard(
             function,
@@ -133,7 +132,7 @@ class TestMCPropagation(unittest.TestCase):
             corr_between=np.ones((2, 2)),
             return_samples=True,
         )
-        npt.assert_allclose(uf, yerr_corr, rtol=0.06)
+        npt.assert_allclose(uf, yerr_corr, atol=0.1)
 
         uf, yvalues, xvalues = prop.propagate_random(
             function,
@@ -143,7 +142,7 @@ class TestMCPropagation(unittest.TestCase):
             corr_between=np.ones((2, 2)),
             return_samples=True,
         )
-        npt.assert_allclose(uf, yerr_corr, rtol=0.06)
+        npt.assert_allclose(uf, yerr_corr, atol=0.1)
 
         uf, yvalues, xvalues = prop.propagate_random(
             function,
@@ -153,7 +152,7 @@ class TestMCPropagation(unittest.TestCase):
             corr_between=np.ones((2, 2)),
             return_samples=True,
         )
-        npt.assert_allclose(uf, yerr_corr, rtol=0.06)
+        npt.assert_allclose(uf, yerr_corr, atol=0.1)
 
     def test_propagate_random_2D(self):
         prop = MCPropagation(20000, parallel_cores=2)
@@ -192,7 +191,7 @@ class TestMCPropagation(unittest.TestCase):
         ufb = prop.propagate_random(
             functionb, xsb, xerrsb, corr_between=np.ones((2, 2))
         )
-        npt.assert_allclose(ufb, yerr_corrb, atol=0.03)
+        npt.assert_allclose(ufb, yerr_corrb, atol=0.06)
 
         ufb, yvaluesb, xvaluesb = prop.propagate_random(
             functionb,
@@ -202,7 +201,7 @@ class TestMCPropagation(unittest.TestCase):
             return_samples=True,
             repeat_dims=1,
         )
-        npt.assert_allclose(ufb, yerr_corrb, atol=0.03)
+        npt.assert_allclose(ufb, yerr_corrb, atol=0.06)
 
     def test_propagate_random_1D_3var(self):
         prop = MCPropagation(20000, parallel_cores=1)
@@ -364,18 +363,17 @@ class TestMCPropagation(unittest.TestCase):
         npt.assert_allclose(ucorrd[0], np.ones_like(ucorrd[0]), atol=0.06)
 
     def test_propagate_cov_1D(self):
-        prop = MCPropagation(20000)
+        prop = MCPropagation(25000)
 
         cov = [
-            util.convert_corr_to_cov(np.eye(len(xerr.flatten())), xerr)
-            for xerr in xerrs
+            cm.convert_corr_to_cov(np.eye(len(xerr.flatten())), xerr) for xerr in xerrs
         ]
         uf, ucorr = prop.propagate_cov(function, xs, cov, return_corr=True)
         npt.assert_allclose(ucorr, np.eye(len(ucorr)), atol=0.06)
         npt.assert_allclose(uf, yerr_uncorr, rtol=0.06)
 
         cov = [
-            util.convert_corr_to_cov(
+            cm.convert_corr_to_cov(
                 np.ones((len(xerr.flatten()), len(xerr.flatten()))) + np.eye(len(xerr)),
                 xerr,
             )
@@ -385,20 +383,19 @@ class TestMCPropagation(unittest.TestCase):
         npt.assert_allclose(uf, yerr_uncorr * 2 ** 0.5, rtol=0.06)
 
         cov = [
-            util.convert_corr_to_cov(np.eye(len(xerr.flatten())), xerr)
-            for xerr in xerrs
+            cm.convert_corr_to_cov(np.eye(len(xerr.flatten())), xerr) for xerr in xerrs
         ]
         uf, ucorr = prop.propagate_cov(
             function, xs, cov, return_corr=True, corr_between=np.ones((2, 2))
         )
         npt.assert_allclose(ucorr, np.eye(len(ucorr)), atol=0.06)
-        npt.assert_allclose(uf, yerr_corr, rtol=0.06)
+        npt.assert_allclose(uf, yerr_corr, atol=0.1)
 
     def test_propagate_cov_2D(self):
         prop = MCPropagation(20000)
 
         covb = [
-            util.convert_corr_to_cov(np.eye(len(xerrb.flatten())), xerrb)
+            cm.convert_corr_to_cov(np.eye(len(xerrb.flatten())), xerrb)
             for xerrb in xerrsb
         ]
         ufb, ucorrb = prop.propagate_cov(functionb, xsb, covb, return_corr=True)
@@ -406,7 +403,7 @@ class TestMCPropagation(unittest.TestCase):
         npt.assert_allclose(ufb, yerr_uncorrb, rtol=0.06)
 
         covb = [
-            util.convert_corr_to_cov(np.eye(len(xerrb[:, 0].flatten())), xerrb[:, 0])
+            cm.convert_corr_to_cov(np.eye(len(xerrb[:, 0].flatten())), xerrb[:, 0])
             for xerrb in xerrsb
         ]
 
@@ -419,7 +416,7 @@ class TestMCPropagation(unittest.TestCase):
         xsb2 = [50, xsb[1]]
         covb = [
             1,
-            util.convert_corr_to_cov(
+            cm.convert_corr_to_cov(
                 np.ones(
                     (len(xerrsb[1][:, 0].flatten()), len(xerrsb[1][:, 0].flatten()))
                 ),
@@ -439,7 +436,7 @@ class TestMCPropagation(unittest.TestCase):
         npt.assert_allclose(ufb, yerr_uncorrb, rtol=0.06)
 
         covb = [
-            util.convert_corr_to_cov(
+            cm.convert_corr_to_cov(
                 np.ones((len(xerrb.flatten()), len(xerrb.flatten())))
                 + np.eye(len(xerrb.flatten())),
                 xerrb,
@@ -450,19 +447,19 @@ class TestMCPropagation(unittest.TestCase):
         npt.assert_allclose(ufb, yerr_uncorrb * 2 ** 0.5, rtol=0.06)
 
         covb = [
-            util.convert_corr_to_cov(np.eye(len(xerrb.flatten())), xerrb)
+            cm.convert_corr_to_cov(np.eye(len(xerrb.flatten())), xerrb)
             for xerrb in xerrsb
         ]
         ufb, ucorrb = prop.propagate_cov(
             functionb, xsb, covb, return_corr=True, corr_between=np.ones((2, 2))
         )
         npt.assert_allclose(ucorrb, np.eye(len(ucorrb)), atol=0.06)
-        npt.assert_allclose(ufb, yerr_corrb, atol=0.03)
+        npt.assert_allclose(ufb, yerr_corrb, atol=0.1)
 
     def test_propagate_cov_1D_3var(self):
         prop = MCPropagation(20000)
         covc = [
-            util.convert_corr_to_cov(np.eye(len(xerrc.flatten())), xerrc)
+            cm.convert_corr_to_cov(np.eye(len(xerrc.flatten())), xerrc)
             for xerrc in xerrsc
         ]
         ufc, ucorrc = prop.propagate_cov(functionc, xsc, covc, return_corr=True)
@@ -470,7 +467,7 @@ class TestMCPropagation(unittest.TestCase):
         npt.assert_allclose(ufc, yerr_uncorrc, rtol=0.06)
 
         covc = [
-            util.convert_corr_to_cov(
+            cm.convert_corr_to_cov(
                 np.ones((len(xerrc.flatten()), len(xerrc.flatten())))
                 + np.eye(len(xerrc)),
                 xerrc,
@@ -481,7 +478,7 @@ class TestMCPropagation(unittest.TestCase):
         npt.assert_allclose(ufc, yerr_uncorrc * 2 ** 0.5, rtol=0.06)
 
         covc = [
-            util.convert_corr_to_cov(np.eye(len(xerrc.flatten())), xerrc)
+            cm.convert_corr_to_cov(np.eye(len(xerrc.flatten())), xerrc)
             for xerrc in xerrsc
         ]
         ufc, ucorrc = prop.propagate_cov(
@@ -494,7 +491,7 @@ class TestMCPropagation(unittest.TestCase):
         prop = MCPropagation(20000)
 
         covd = [
-            util.convert_corr_to_cov(np.eye(len(xerrd.flatten())), xerrd)
+            cm.convert_corr_to_cov(np.eye(len(xerrd.flatten())), xerrd)
             for xerrd in xerrsd
         ]
         ufd, ucorrd, corr_out = prop.propagate_cov(
@@ -505,7 +502,7 @@ class TestMCPropagation(unittest.TestCase):
         npt.assert_allclose(ufd[1], yerr_uncorrd[1], rtol=0.06)
 
         covd = [
-            util.convert_corr_to_cov(
+            cm.convert_corr_to_cov(
                 np.ones((len(xerrd.flatten()), len(xerrd.flatten()))), xerrd
             )
             for xerrd in xerrsd
